﻿using System;
using System.Text;
using System.Numerics;
using System.Threading;
using System.Threading.Tasks;
using System.Collections.Generic;
using Nethereum.Web3;
using Nethereum.Web3.Accounts;
using Nethereum.Web3.Accounts.Managed;
using Nethereum.Contracts;
using Nethereum.Contracts.CQS;
using Nethereum.Contracts.Extensions;
using Nethereum.Util;
using Nethereum.HdWallet;
using Nethereum.Hex.HexConvertors.Extensions;
using ServerSettings;


class Program
{
    static void Main(string[] args)
    {
        Console.WriteLine("");
        Console.WriteLine("");
        RopstenContractInteraction().Wait();
        Console.WriteLine("");
        Console.WriteLine("");
        GetAccountBalance().Wait();
        Console.WriteLine("");
        Console.WriteLine("");
        QuorumInteraction().Wait(); 
        Console.WriteLine("");
        Console.WriteLine("");

    }

    static async Task<Account> GetAccount()
    {
        WalletSettings ws = new WalletSettings();

        var words = ws.Words;
        var password = ws.Password;
        
        var account = new Wallet(words, password).GetAccount(0);

        return account;
    }

    static async Task TransferEther()
    {
        MetamaskSettings ms = new MetamaskSettings();
        RopstenSettings rp = new RopstenSettings();

        var account = await GetAccount();
        var web3 = new Web3(account, rp.Url);
        var balance = await web3.Eth.GetBalance.SendRequestAsync(account.Address);
        var balance2 = await web3.Eth.GetBalance.SendRequestAsync(ms.PublicKey);

        Console.WriteLine("The account balance of Sender " + account.Address + "is: " + balance.Value);
        Console.WriteLine("The account balance of Receiver " + ms.PublicKey + " is: " + balance2.Value);

        var toAddress = ms.PublicKey;
        var transactionReceipt = await web3.Eth.GetEtherTransferService()
            .TransferEtherAndWaitForReceiptAsync(toAddress, 0.11m, 2);
        Console.WriteLine($"Transaction {transactionReceipt.TransactionHash} for amount of 0.11 Ether completed");
    }

    static async Task RopstenContractInteraction()
    {
        RopstenSettings rp = new RopstenSettings();

        var account = await GetAccount();
        var web3 = new Web3(account, rp.Url);
        var abi = rp.ContractAbi;
        var contract = web3.Eth.GetContract(abi, rp.ContractAddress);

        var functionSet = contract.GetFunction("getHash");
        var result = await functionSet.CallAsync<string>(22);
        Console.WriteLine("getHash(22): " + result);
    }

    static async Task GetAccountBalance()
    {
        RopstenSettings rp = new RopstenSettings();
        MetamaskSettings ms = new MetamaskSettings();

        /* Get Account balance*/
        var publicKey = ms.PublicKey;

        var web3 = new Web3(rp.Url);

        var balance = await web3.Eth.GetBalance.SendRequestAsync(publicKey);
        var etherAmount = Web3.Convert.FromWei(balance.Value);
        Console.WriteLine($"Account Balance of {ms.PublicKey} on Ropsten Test Network");
        Console.WriteLine($"Balance in Wei: {balance.Value}");
        Console.WriteLine($"Balance in Ether: {etherAmount}");
    }

    static async Task QuorumInteraction() 
    {
        QuorumSettings qs = new QuorumSettings();
        WalletSettings ws = new WalletSettings();

        var account = await GetAccount();
        var managedAccount = new ManagedAccount(account.Address, ws.Password);
        var web3Managed = new Web3(managedAccount, qs.UrlWithAccessKey);

        try
        {
            var blockNumber = await web3Managed.Eth.Blocks.GetBlockNumber.SendRequestAsync();
            Console.WriteLine("Current Block Number: " + blockNumber);
            
            var balance = await web3Managed.Eth.GetBalance.SendRequestAsync(account.Address);
            Console.WriteLine("Account Balance on Azure Quorum for "+ account.Address +": " + balance);
            
            var contract = web3Managed.Eth.GetContract(qs.ContractAbi, qs.ContractAddress);

            var functionSet = contract.GetFunction("getLatestFileIndex");
            var result = await functionSet.CallAsync<int>();
            
            Console.WriteLine("getLatestFileIndex(): " + result);

        }
        catch (Exception e)
        {
            Console.WriteLine(e);
        }

    }
}